@@ -25,7 +25,6 @@ else:
     has_theme = True
 
 
-sys.path.insert(0, os.path.abspath('../..'))
 # -- General configuration ----------------------------------------------------
 
 # Add any Sphinx extension module names here, as strings. They can be
@@ -34,6 +33,7 @@ extensions = [
     'sphinx.ext.autodoc',
     # 'sphinx.ext.intersphinx',
     'reno.sphinxext',
+    'reno._exts.show_reno_config',
 ]
 
 if has_theme:
@@ -42,8 +42,8 @@ if has_theme:
 
 # openstackdocstheme options
 repository_name = 'openstack/reno'
-bug_project = 'reno'
-bug_tag = ''
+bug_project = '933'
+bug_tag = 'docs'
 html_last_updated_fmt = '%Y-%m-%d %H:%M'
 
 # autodoc generation is a bit aggressive and a nuisance when doing heavy
