@@ -32,7 +32,7 @@ _query_args = [
           help='the branch to scan, defaults to the current')),
     (('--collapse-pre-releases',),
      dict(action='store_true',
-          default=config.Config.get_default('collapse_pre_releases'),
+          default=None,
           help='combine pre-releases with their final release')),
     (('--no-collapse-pre-releases',),
      dict(action='store_false',
@@ -42,12 +42,12 @@ _query_args = [
      dict(default=None,
           help='stop when this version is reached in the history')),
     (('--ignore-cache',),
-     dict(default=False,
+     dict(default=None,
           action='store_true',
           help='if there is a cache file present, do not use it')),
     (('--stop-at-branch-base',),
      dict(action='store_true',
-          default=True,
+          default=None,
           dest='stop_at_branch_base',
           help='stop scanning when the branch meets master')),
     (('--no-stop-at-branch-base',),
@@ -88,6 +88,9 @@ def main(argv=sys.argv[1:]):
     )
     subparsers = parser.add_subparsers(
         title='commands',
+        description='valid commands',
+        dest='command',
+        help='additional help',
     )
 
     do_new = subparsers.add_parser(
@@ -99,6 +102,10 @@ def main(argv=sys.argv[1:]):
         action='store_true',
         help='Edit note after its creation (require EDITOR env variable)',
     )
+    do_new.add_argument(
+        '--from-template',
+        help='Template to get the release note from.',
+    )
     do_new.add_argument(
         'slug',
         help='descriptive title of note (keep it short)',
@@ -187,12 +194,17 @@ def main(argv=sys.argv[1:]):
     do_linter.set_defaults(func=linter.lint_cmd)
 
     args = parser.parse_args(argv)
-    conf = config.Config(args.reporoot, args.relnotesdir)
-    conf.override_from_parsed_args(args)
+    # no arguments, print help messaging, then exit with error(1)
+    if not args.command:
+        parser.print_help()
+        return 1
 
     logging.basicConfig(
         level=args.verbosity,
         format='%(message)s',
     )
 
+    conf = config.Config(args.reporoot, args.relnotesdir)
+    conf.override_from_parsed_args(args)
+
     return args.func(args, conf)
