@@ -252,6 +252,10 @@ class _ChangeTracker(object):
         self.last_name_by_id = {}
         # Remember uniqueids that have had files deleted.
         self.uniqueids_deleted = set()
+        # Remember files that are changed but not explicitly added so
+        # when we do see an add we can use the more recent tracking
+        # info and if we don't see the add we know to ignore the file.
+        self.seen_but_not_added = {}
 
     def _common(self, uniqueid, sha, version):
         if version not in self.versions:
@@ -284,12 +288,24 @@ class _ChangeTracker(object):
             )
             return
 
-        # A note is being added in this commit. If we have
-        # not seen it before, it was added here and never
-        # changed.
-        if uniqueid not in self.last_name_by_id:
+        if uniqueid in self.seen_but_not_added:
+            # The note was seen for a modify operation already but
+            # this is where it was added. We want to remember where
+            # the modification happened, because that came earlier in
+            # the scan (and therefore later in the history).
+            filename, sha = self.seen_but_not_added[uniqueid]
             self.last_name_by_id[uniqueid] = (filename, sha)
             LOG.info(
+                '%s: copying data for %s from commit %s',
+                uniqueid, filename, sha,
+            )
+            del self.seen_but_not_added[uniqueid]
+        elif uniqueid not in self.last_name_by_id:
+            # The note was added already and we want to keep that
+            # other reference because it came earlier in the scan (and
+            # therefore later in the history).
+            self.last_name_by_id[uniqueid] = (filename, sha)
+            LOG.debug(
                 '%s: new %s in commit %s',
                 uniqueid, filename, sha,
             )
@@ -299,7 +315,7 @@ class _ChangeTracker(object):
                 uniqueid,
             )
 
-    def rename(self, filename, sha, version):
+    def _change(self, filename, sha, version):
         uniqueid = _get_unique_id(filename)
         self._common(uniqueid, sha, version)
 
@@ -313,42 +329,19 @@ class _ChangeTracker(object):
             )
             return
 
+        if uniqueid in self.last_name_by_id:
+            LOG.debug('%s: already added', uniqueid)
+            to_update = self.last_name_by_id
+        else:
+            LOG.debug('%s: seen but not added', uniqueid)
+            to_update = self.seen_but_not_added
+
         # The file is being renamed. We may have seen it
         # before, if there were subsequent modifications,
         # so only store the name information if it is not
         # there already.
-        if uniqueid not in self.last_name_by_id:
-            self.last_name_by_id[uniqueid] = (filename, sha)
-            LOG.info(
-                '%s: update to %s in commit %s',
-                uniqueid, filename, sha,
-            )
-        else:
-            LOG.debug(
-                '%s: renamed file already known with the new name',
-                uniqueid,
-            )
-
-    def modify(self, filename, sha, version):
-        uniqueid = _get_unique_id(filename)
-        self._common(uniqueid, sha, version)
-
-        # If we have recorded that a UID was deleted, that
-        # means that was the last change made to the file and
-        # we can ignore it.
-        if uniqueid in self.uniqueids_deleted:
-            LOG.debug(
-                '%s: has already been deleted, ignoring this change',
-                uniqueid,
-            )
-            return
-
-        # An existing file is being modified. We may have
-        # seen it before, if there were subsequent
-        # modifications, so only store the name
-        # information if it is not there already.
-        if uniqueid not in self.last_name_by_id:
-            self.last_name_by_id[uniqueid] = (filename, sha)
+        if uniqueid not in to_update:
+            to_update[uniqueid] = (filename, sha)
             LOG.info(
                 '%s: update to %s in commit %s',
                 uniqueid, filename, sha,
@@ -359,6 +352,12 @@ class _ChangeTracker(object):
                 uniqueid,
             )
 
+    def rename(self, filename, sha, version):
+        self._change(filename, sha, version)
+
+    def modify(self, filename, sha, version):
+        self._change(filename, sha, version)
+
     def delete(self, filename, sha, version):
         uniqueid = _get_unique_id(filename)
         self._common(uniqueid, sha, version)
@@ -514,6 +513,11 @@ class Scanner(object):
             self.conf.branch_name_re,
             flags=re.VERBOSE | re.UNICODE,
         )
+        self.branch_name_prefix = self.conf.branch_name_prefix
+        self.closed_branch_tag_re = re.compile(
+            self.conf.closed_branch_tag_re,
+            flags=re.VERBOSE | re.UNICODE,
+        )
         self._ignore_uids = set(
             _get_unique_id(fn)
             for fn in self.conf.ignore_notes
@@ -532,7 +536,17 @@ class Scanner(object):
                 # name exists on the 'origin' remote.
                 'refs/remotes/origin/' + name,
             ]
+            # If the reference points explicitly to the origin remote,
+            # but that remote isn't present (as it won't be when zuul
+            # configures the repo in CI), then we want the shortened
+            # form of the reference. We put this option last in the
+            # list because we want the more explicit name to be used
+            # when someone is running reno locally with a more
+            # standard git configuration.
+            if name.startswith('origin/'):
+                candidates.append('refs/heads/' + name.partition('/')[-1])
             for ref in candidates:
+                LOG.debug('looking for ref {!r} as {!r}'.format(name, ref))
                 key = ref.encode('utf-8')
                 if key in self._repo.refs:
                     sha = self._repo.refs[key]
@@ -542,6 +556,8 @@ class Scanner(object):
                         # signed tags point to the signature and we
                         # need to dereference it to get to the commit.
                         sha = o.object[1]
+                    LOG.info('found ref {!r} as {!r} at {}'.format(
+                        name, ref, sha))
                     return sha
             # If we end up here we didn't find any of the candidates.
             raise ValueError('Unknown reference {!r}'.format(name))
@@ -807,6 +823,18 @@ class Scanner(object):
             elif r.startswith('refs/heads/'):
                 name = r[11:]
             if name and self.branch_name_re.search(name):
+                LOG.debug('branch name %s', name)
+                branch_names.add(name)
+                continue
+            if not r.startswith('refs/tags/'):
+                continue
+            # See if the ref is a closed branch tag.
+            name = r.rpartition('/')[-1]
+            match = self.closed_branch_tag_re.search(name)
+            if match:
+                name = self.branch_name_prefix + match.group(1)
+                LOG.debug('closed branch tag %s becomes %s',
+                          r.rpartition('/')[-1], name)
                 branch_names.add(name)
         return list(sorted(branch_names))
 
@@ -901,9 +929,14 @@ class Scanner(object):
         collapse_pre_releases = self.conf.collapse_pre_releases
         stop_at_branch_base = self.conf.stop_at_branch_base
 
-        LOG.info('scanning %s/%s (branch=%s earliest_version=%s)',
-                 reporoot.rstrip('/'), notesdir.lstrip('/'),
-                 branch or '*current*', earliest_version)
+        LOG.info(
+            ('scanning %s/%s '
+             '(branch=%s earliest_version=%s collapse_pre_releases=%s)'),
+            reporoot.rstrip('/'), notesdir.lstrip('/'),
+            branch or '*current*',
+            earliest_version,
+            collapse_pre_releases,
+        )
 
         # Determine the current version, which might be an unreleased or
         # dev version if there are unreleased commits at the head of the
@@ -971,6 +1004,7 @@ class Scanner(object):
             # base of the branch, which involves a bit of searching.
             LOG.debug('determining earliest_version from branch')
             branch_base = self._get_branch_base(branch)
+            LOG.debug('branch base %s', branch_base)
             scan_stop_tag = self._find_scan_stop_point(
                 branch_base, versions_by_date,
                 collapse_pre_releases, branch)
@@ -979,6 +1013,8 @@ class Scanner(object):
             else:
                 idx = versions_by_date.index(scan_stop_tag)
                 earliest_version = versions_by_date[idx - 1]
+                LOG.debug('using version before %s as scan stop point',
+                          scan_stop_tag)
             if earliest_version and collapse_pre_releases:
                 if self.pre_release_tag_re.search(earliest_version):
                     # The earliest version won't actually be the pre-release
