@@ -9,14 +9,15 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from __future__ import unicode_literals
 
-import logging
 import os.path
 
 from docutils import nodes
 from docutils.parsers import rst
 from docutils.parsers.rst import directives
 from docutils import statemachine
+from sphinx.util import logging
 from sphinx.util.nodes import nested_parse_with_titles
 
 from dulwich import repo
@@ -25,11 +26,16 @@ from reno import defaults
 from reno import formatter
 from reno import loader
 
+LOG = logging.getLogger(__name__)
+
 
 class ReleaseNotesDirective(rst.Directive):
 
     has_content = True
 
+    # FIXME(dhellmann): We should be able to build this information
+    # from the configuration options so we don't have to edit it
+    # manually when we add new options.
     option_spec = {
         'branch': directives.unchanged,
         'reporoot': directives.unchanged,
@@ -40,15 +46,10 @@ class ReleaseNotesDirective(rst.Directive):
         'earliest-version': directives.unchanged,
         'stop-at-branch-base': directives.flag,
         'ignore-notes': directives.unchanged,
+        'unreleased-version-title': directives.unchanged,
     }
 
     def run(self):
-        env = self.state.document.settings.env
-        app = env.app
-
-        def info(msg):
-            app.info('[reno] %s' % (msg,))
-
         title = ' '.join(self.content)
         branch = self.options.get('branch')
         reporoot_opt = self.options.get('reporoot', '.')
@@ -77,6 +78,10 @@ class ReleaseNotesDirective(rst.Directive):
         if 'earliest-version' in self.options:
             opt_overrides['earliest_version'] = self.options.get(
                 'earliest-version')
+        if 'unreleased-version-title' in self.options:
+            opt_overrides['unreleased_version_title'] = self.options.get(
+                'unreleased-version-title')
+
         if branch:
             opt_overrides['branch'] = branch
         if ignore_notes:
@@ -84,9 +89,9 @@ class ReleaseNotesDirective(rst.Directive):
         conf.override(**opt_overrides)
 
         notesdir = os.path.join(relnotessubdir, conf.notesdir)
-        info('scanning %s for %s release notes' %
-             (os.path.join(conf.reporoot, notesdir),
-              branch or 'current branch'))
+        LOG.info('scanning %s for %s release notes' % (
+                 os.path.join(conf.reporoot, notesdir),
+                 branch or 'current branch'))
 
         ldr = loader.Loader(conf)
         if version_opt is not None:
@@ -96,17 +101,19 @@ class ReleaseNotesDirective(rst.Directive):
             ]
         else:
             versions = ldr.versions
-        info('got versions %s' % (versions,))
+        LOG.info('got versions %s' % (versions,))
         text = formatter.format_report(
             ldr,
             conf,
             versions,
             title=title,
+            branch=branch,
         )
         source_name = '<%s %s>' % (__name__, branch or 'current branch')
         result = statemachine.ViewList()
-        for line in text.splitlines():
-            result.append(line, source_name)
+        for line_num, line in enumerate(text.splitlines(), 1):
+            LOG.debug('%4d: %s', line_num, line)
+            result.append(line, source_name, line_num)
 
         node = nodes.section()
         node.document = self.state.document
@@ -116,8 +123,3 @@ class ReleaseNotesDirective(rst.Directive):
 
 def setup(app):
     app.add_directive('release-notes', ReleaseNotesDirective)
-
-    logging.basicConfig(
-        level=logging.INFO,
-        format='[%(name)s] %(message)s',
-    )
