@@ -273,6 +273,20 @@ class BasicTest(Base):
             results,
         )
 
+    def test_tag_with_v_prefix(self):
+        filename = self._add_notes_file()
+        self.repo.git('tag', '-s', '-m', 'tag with v prefix', 'v1.0.0')
+        self.scanner = scanner.Scanner(self.c)
+        raw_results = self.scanner.get_notes_by_version()
+        results = {
+            k: [f for (f, n) in v]
+            for (k, v) in raw_results.items()
+        }
+        self.assertEqual(
+            {'v1.0.0': [filename]},
+            results,
+        )
+
     def test_note_commit_after_tag(self):
         self._make_python_package()
         self.repo.git('tag', '-s', '-m', 'first tag', '1.0.0')
@@ -876,6 +890,23 @@ class PreReleaseTest(Base):
             results,
         )
 
+    def test_tag_with_v_prefix(self):
+        self._make_python_package()
+        self.repo.git('tag', '-s', '-m', 'first tag', 'v1.0.0.0a1')
+        f1 = self._add_notes_file('slug1')
+        self.repo.git('tag', '-s', '-m', 'first tag', 'v1.0.0.0a2')
+        self.scanner = scanner.Scanner(self.c)
+        raw_results = self.scanner.get_notes_by_version()
+        results = {
+            k: [f for (f, n) in v]
+            for (k, v) in raw_results.items()
+        }
+        self.assertEqual(
+            {'v1.0.0.0a2': [f1],
+             },
+            results,
+        )
+
     def test_collapse(self):
         files = []
         self._make_python_package()
@@ -1254,7 +1285,7 @@ class BranchTest(Base):
         self.repo.git('tag', '-s', '-m', 'first tag', '1.0.0')
         self.f2 = self._add_notes_file('slug2')
         self.repo.git('tag', '-s', '-m', 'first tag', '2.0.0')
-        self._add_notes_file('slug3')
+        self.f3 = self._add_notes_file('slug3')
         self.repo.git('tag', '-s', '-m', 'first tag', '3.0.0')
 
     def test_files_current_branch(self):
@@ -1613,6 +1644,33 @@ class BranchTest(Base):
         self.assertIsNotNone(head2)
         self.assertEqual(head1, head2)
 
+    def test_modify_old_branch_note_on_master(self):
+        # Modify a note from a stable branch on master and ensure that
+        # the note does not appear in the scanner output from master.
+        # This should replicate the problem described in bug #1682796
+        self.repo.git('checkout', '2.0.0')
+        self.repo.git('branch', 'stable/2')
+        self.repo.git('checkout', 'master')
+        with open(os.path.join(self.reporoot, self.f1), 'w') as f:
+            f.write('new file contents')
+        self.repo.commit('update %s' % self.f1)
+        self.c.override(
+            earliest_version=None,
+        )
+        self.scanner = scanner.Scanner(self.c)
+        raw_results = self.scanner.get_notes_by_version()
+        results = {
+            k: [f for (f, n) in v]
+            for (k, v) in raw_results.items()
+        }
+        self.assertEqual(
+            {
+                '2.0.0': [self.f2],
+                '3.0.0': [self.f3],
+            },
+            results,
+        )
+
 
 class ScanStopPointPrereleaseVersionsTest(Base):
 
@@ -1794,6 +1852,26 @@ class GetRefTest(Base):
         expected = self.scanner._repo.head()
         self.assertEqual(expected, ref)
 
+    def test_stable_branch(self):
+        self.scanner = scanner.Scanner(self.c)
+        ref = self.scanner._get_ref('stable/foo')
+        expected = self.scanner._repo.head()
+        self.assertEqual(expected, ref)
+
+    def test_stable_branch_with_origin_prefix(self):
+        self.scanner = scanner.Scanner(self.c)
+        ref = self.scanner._get_ref('origin/stable/foo')
+        expected = self.scanner._repo.head()
+        self.assertEqual(expected, ref)
+
+    def test_no_such_value(self):
+        self.scanner = scanner.Scanner(self.c)
+        self.assertRaises(
+            ValueError,
+            self.scanner._get_ref,
+            'missing/remote',
+        )
+
 
 class TagsTest(Base):
 
@@ -2207,3 +2285,188 @@ class AggregateChangesTest(Base):
              ('%016x' % n, 'modify', old_name, 'commit-id')],
             results,
         )
+
+
+class ChangeTrackerTest(base.TestCase):
+
+    def setUp(self):
+        super(ChangeTrackerTest, self).setUp()
+        self.changes = scanner._ChangeTracker()
+        basename = '%s-%016x.yaml' % ('slug', 1)
+        self.filename = os.path.join('releasenotes', 'notes', basename)
+        self.filename2 = self.filename.replace('slug', 'guls')
+        self.uniqueid = scanner._get_unique_id(self.filename)
+        self.fake_logger = self.useFixture(
+            fixtures.FakeLogger(
+                format='%(levelname)8s %(name)s %(message)s',
+                level=logging.DEBUG,
+                nuke_handlers=True,
+            )
+        )
+
+    def test_add(self):
+        self.changes.add(self.filename, 'sha1', 'version')
+        self.assertEqual(
+            {},
+            self.changes.seen_but_not_added,
+        )
+        self.assertEqual(
+            ['version'],
+            self.changes.versions,
+        )
+        self.assertEqual(
+            'version',
+            self.changes.earliest_seen[self.uniqueid],
+        )
+        self.assertEqual(
+            {self.uniqueid: (self.filename, 'sha1')},
+            self.changes.last_name_by_id,
+        )
+        self.assertEqual(
+            set(),
+            self.changes.uniqueids_deleted,
+        )
+
+    def test_modify_with_add(self):
+        self.changes.modify(self.filename, 'sha2', 'version2')
+        self.changes.add(self.filename, 'sha1', 'version1')
+        self.assertEqual(
+            {},
+            self.changes.seen_but_not_added,
+        )
+        self.assertEqual(
+            ['version2', 'version1'],
+            self.changes.versions,
+        )
+        self.assertEqual(
+            'version1',
+            self.changes.earliest_seen[self.uniqueid],
+        )
+        self.assertEqual(
+            {self.uniqueid: (self.filename, 'sha2')},
+            self.changes.last_name_by_id,
+        )
+        self.assertEqual(
+            set(),
+            self.changes.uniqueids_deleted,
+        )
+
+    def test_modify_without_add(self):
+        self.changes.modify(self.filename, 'sha2', 'version2')
+        self.assertEqual(
+            {self.uniqueid: (self.filename, 'sha2')},
+            self.changes.seen_but_not_added,
+        )
+        self.assertEqual(
+            ['version2'],
+            self.changes.versions,
+        )
+        self.assertEqual(
+            'version2',
+            self.changes.earliest_seen[self.uniqueid],
+        )
+        self.assertEqual(
+            {},
+            self.changes.last_name_by_id,
+        )
+        self.assertEqual(
+            set(),
+            self.changes.uniqueids_deleted,
+        )
+
+    def test_rename_with_add(self):
+        self.changes.rename(self.filename2, 'sha2', 'version2')
+        self.changes.add(self.filename, 'sha1', 'version1')
+        self.assertEqual(
+            {},
+            self.changes.seen_but_not_added,
+        )
+        self.assertEqual(
+            ['version2', 'version1'],
+            self.changes.versions,
+        )
+        self.assertEqual(
+            'version1',
+            self.changes.earliest_seen[self.uniqueid],
+        )
+        self.assertEqual(
+            {self.uniqueid: (self.filename2, 'sha2')},
+            self.changes.last_name_by_id,
+        )
+        self.assertEqual(
+            set(),
+            self.changes.uniqueids_deleted,
+        )
+
+    def test_rename_without_add(self):
+        self.changes.rename(self.filename2, 'sha2', 'version2')
+        self.assertEqual(
+            {self.uniqueid: (self.filename2, 'sha2')},
+            self.changes.seen_but_not_added,
+        )
+        self.assertEqual(
+            ['version2'],
+            self.changes.versions,
+        )
+        self.assertEqual(
+            'version2',
+            self.changes.earliest_seen[self.uniqueid],
+        )
+        self.assertEqual(
+            {},
+            self.changes.last_name_by_id,
+        )
+        self.assertEqual(
+            set(),
+            self.changes.uniqueids_deleted,
+        )
+
+
+class GetSeriesBranchesTest(Base):
+
+    def setUp(self):
+        super(GetSeriesBranchesTest, self).setUp()
+        self.repo.add_file('test.txt')
+
+    def test_none(self):
+        self.scanner = scanner.Scanner(self.c)
+        self.assertEqual(
+            [],
+            self.scanner._get_series_branches(),
+        )
+
+    def test_real_branches_sorted_names(self):
+        self.repo.git(
+            'checkout', '-b', 'stable/a',
+        )
+        self.repo.git(
+            'checkout', '-b', 'stable/b',
+        )
+        self.scanner = scanner.Scanner(self.c)
+        self.assertEqual(
+            ['stable/a', 'stable/b'],
+            self.scanner._get_series_branches(),
+        )
+
+    def test_eol_tag(self):
+        self.repo.git(
+            'tag', '-s', '-m', 'closed branch', 'a-eol',
+        )
+        self.scanner = scanner.Scanner(self.c)
+        self.assertEqual(
+            ['stable/a'],
+            self.scanner._get_series_branches(),
+        )
+
+    def test_mix_tag_and_branch(self):
+        self.repo.git(
+            'tag', '-s', '-m', 'closed branch', 'a-eol',
+        )
+        self.repo.git(
+            'checkout', '-b', 'stable/b',
+        )
+        self.scanner = scanner.Scanner(self.c)
+        self.assertEqual(
+            ['stable/a', 'stable/b'],
+            self.scanner._get_series_branches(),
+        )
